@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 ###
-### $Release: 0.12.1 $
+### $Release: 0.13.0 $
 ### $Copyright: copyright(c) 2010-2014 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
@@ -97,6 +97,41 @@ except (ImportError, SyntaxError):
             self._data = to_binary(data)
         data = property(get_data, set_data)
 
+try:
+    from requests.models import Response as RequestsResponse
+except (ImportError, SyntaxError):
+    class RequestsResponse(object):
+        def __int__(self):
+            self.status_code = None
+            self.reason = None
+            self.headers = {}
+            self._content = None
+        @property
+        def content(self):
+            return self._content
+        @property
+        def text(self):
+            return self._content.decode('utf-8')
+finally:
+    def _setUp(self, status=200, headers=None):
+        if isinstance(status, int):
+            self.status_code = status
+            self.reason = {200:'OK', 201:'Created', 301: 'Moved Permanently', 302:'Found'}.get(status)
+        elif isinstance(status, str):
+            code, msg = str.split(' ', 1)
+            self.status_code = code
+            self.reason = msg
+        else:
+            raise TypeError
+        self.headers = headers or {}
+        self._content = to_binary("")
+        return self
+    assert not hasattr(RequestsResponse, '_setUp')
+    RequestsResponse._setUp = _setUp
+    del _setUp
+
+from oktest.web import WSGIResponse as OktestWSGIResponse
+
 
 def _set_body(response, body):
     if hasattr(response, 'body'):
@@ -104,8 +139,13 @@ def _set_body(response, body):
         response.text = to_unicode(body)
     elif hasattr(response, 'data'):
         response.data = body
+    elif hasattr(response, '_content'):
+        response._content = to_binary(body)
+    elif hasattr(response, 'body_binary'):
+        response._body_binary  = to_binary(body)
+        response._body_unicode = to_unicode(body)
     else:
-        raise Error
+        raise Exception
 
 def _set_ctype(response, content_type):
     if hasattr(response, 'content_type'):
@@ -127,7 +167,9 @@ def with_response_class(func):
     def newfunc(self):
         #for klass in [WebObResponse]:
         #for klass in [WerkzeugResponse]:
-        for klass in [WebObResponse, WerkzeugResponse]:
+        #for klass in [RequestsResponse]:
+        #for klass in [OktestWSGIResponse]:
+        for klass in [WebObResponse, WerkzeugResponse, RequestsResponse, OktestWSGIResponse]:
             func(self, klass)
     newfunc.__name__ = func.__name__
     newfunc.__doc__  = func.__doc__
@@ -146,13 +188,14 @@ class ResponseAssertionObject_TC(unittest.TestCase):
     @with_response_class
     def test_is_response(self, Response):
         response = Response()
+        if hasattr(response, '_setUp'): response._setUp()
         ret = ok (response).is_response(200); ret != None
         ok (ret).is_a(oktest.ResponseAssertionObject)
         #
         try:
             ok (response).is_response(200)
-            ok (response).is_response('200 OK')
-            ok (response).is_response((200, 201))
+            #ok (response).is_response('200 OK')
+            #ok (response).is_response((200, 201))
         except:
             assert False, "failed"
         #
@@ -200,21 +243,34 @@ Unexpected content-type value.
 
     @with_response_class
     def test_status_ok(self, Response):
+        if Response is OktestWSGIResponse:
+            status = '302 Found'
+        else:
+            status = 302
         try:
-            ok (Response())._resp.status(200)
-            ok (Response(status=302))._resp.status(302)
-            ok (Response(status=302))._resp.status((301, 302))
+            if hasattr(Response, '_setUp'):
+                ok (Response()._setUp())._resp.status(200)
+                ok (Response()._setUp(status))._resp.status(302)
+                ok (Response()._setUp(status))._resp.status((301, 302))
+                pass
+            else:
+                ok (Response())._resp.status(200)
+                ok (Response(status=status))._resp.status(302)
+                ok (Response(status=status))._resp.status((301, 302))
         except:
             assert False, "failed"
 
     @with_response_class
     def test_status_ok_returns_self(self, Response):
-        respobj = ok (Response())._resp
-        assert respobj.status(200) is respobj
+        resp = Response()
+        if hasattr(resp, '_setUp'): resp._setUp()
+        ret = ok (resp)._resp
+        assert ret.status(200) is ret
 
     @with_response_class
     def test_status_NG(self, Response):
         response = Response()
+        if hasattr(response, '_setUp'): response._setUp()
         _set_body(response, to_binary('{"status": "OK"}'))
         expected_errmsg = r"""
 Response status 200 == 201: failed.
@@ -241,6 +297,7 @@ b'{"status": "OK"}'
     @with_response_class
     def test_cont_type_ok(self, Response):
         resp = Response()
+        if hasattr(resp, '_setUp'): resp._setUp()
         _set_ctype(resp, 'image/jpeg')
         try:
             ok (resp)._resp.cont_type('image/jpeg')
@@ -251,6 +308,7 @@ b'{"status": "OK"}'
     @with_response_class
     def test_cont_type_ok_returns_self(self, Response):
         resp = Response()
+        if hasattr(resp, '_setUp'): resp._setUp()
         _set_ctype(resp, 'image/jpeg')
         respobj = ok (resp)._resp
         assert respobj.cont_type('image/jpeg') is respobj
@@ -259,6 +317,7 @@ b'{"status": "OK"}'
     @with_response_class
     def test_cont_type_NG(self, Response):
         resp = Response()
+        if hasattr(resp, '_setUp'): resp._setUp()
         #
         _set_ctype(resp, 'image/jpeg')
         expected_errmsg = r"""
@@ -282,6 +341,7 @@ Unexpected content-type value (not matched to pattern).
     @with_response_class
     def test_header_ok(self, Response):
         response = Response()
+        if hasattr(response, '_setUp'): response._setUp()
         response.headers['Location'] = '/'
         try:
             ok (response)._resp.header('Location', '/')
@@ -292,6 +352,7 @@ Unexpected content-type value (not matched to pattern).
     @with_response_class
     def test_header_ok_returns_self(self, Response):
         response = Response()
+        if hasattr(response, '_setUp'): response._setUp()
         response.headers['Location'] = '/'
         respobj = ok (response)._resp
         assert respobj.header('Location', '/') is respobj
@@ -299,6 +360,7 @@ Unexpected content-type value (not matched to pattern).
     @with_response_class
     def test_header_NG(self, Response):
         response = Response()
+        if hasattr(response, '_setUp'): response._setUp()
         response.headers['Location'] = '/'
         expected_errmsg = r"""
 Response header 'Location' is unexpected value.
@@ -325,6 +387,7 @@ Response header 'Location' should not be set : failed.
     @with_response_class
     def test_body_ok(self, Response):
         response = Response()
+        if hasattr(response, '_setUp'): response._setUp()
         _set_body(response, '<h1>Hello</h1>')
         try:
             ok (response)._resp.body('<h1>Hello</h1>')
@@ -336,6 +399,7 @@ Response header 'Location' should not be set : failed.
     @with_response_class
     def test_body_NG(self, Response):
         response = Response()
+        if hasattr(response, '_setUp'): response._setUp()
         _set_body(response, to_binary('<h1>Hello</h1>'))
         #
         expected_msg = r"""
@@ -364,6 +428,7 @@ Response body failed to match to expected pattern.
     @with_response_class
     def test_json_ok(self, Response):
         response = Response()
+        if hasattr(response, '_setUp'): response._setUp()
         content_types = [
             'application/json',
             'application/json;charset=utf8',
@@ -382,6 +447,7 @@ Response body failed to match to expected pattern.
     @with_response_class
     def test_json_ok_returns_self(self, Response):
         response = Response()
+        if hasattr(response, '_setUp'): response._setUp()
         _set_ctype(response, 'application/json')
         _set_body(response, to_binary('{"status": "OK"}'))
         respobj = ok (response)._resp
@@ -390,6 +456,7 @@ Response body failed to match to expected pattern.
     @with_response_class
     def test_json_NG_when_content_type_is_empty(self, Response):
         response = Response()
+        if hasattr(response, '_setUp'): response._setUp()
         _set_body(response, to_binary('{"status": "OK"}'))
         _set_ctype(response, '')
         @be_failed("Content-Type is not set.")
@@ -399,6 +466,7 @@ Response body failed to match to expected pattern.
     @with_response_class
     def test_json_NG_when_content_type_is_not_json_type(self, Response):
         response = Response()
+        if hasattr(response, '_setUp'): response._setUp()
         _set_body(response, to_binary('''{"status": "OK"}'''))
         _set_ctype(response, 'text/html; charset=UTF-8')
         expected = ("Content-Type should be 'application/json' : failed.\n"
@@ -410,6 +478,7 @@ Response body failed to match to expected pattern.
     @with_response_class
     def test_json_NG_when_json_data_is_different(self, Response):
         response = Response()
+        if hasattr(response, '_setUp'): response._setUp()
         _set_body(response, to_binary('''{"status": "OK"}'''))
         _set_ctype(response, 'application/json')
         expected = r"""
@@ -427,6 +496,30 @@ Responsed JSON is different from expected data.
         def _():
             ok (response)._resp.json({"status": "ok"})
 
+    def test_raises_UnsupportedResponseObjectError(self):
+        if python2:
+            errmsg = "'foo': failed to get response %s; <type 'str'> is unsupported response class in oktest."
+        else:
+            errmsg = "'foo': failed to get response %s; <class 'str'> is unsupported response class in oktest."
+        #
+        def fn(): ok ("foo").is_response(200)
+        ok (fn).raises(oktest.UnsupportedResponseObjectError, errmsg % 'status code')
+        #
+        def fn(): ok ("foo")._resp.status(200)
+        ok (fn).raises(oktest.UnsupportedResponseObjectError, errmsg % 'status code')
+        #
+        def fn(): ok ("foo")._resp.cont_type('text/plain')
+        ok (fn).raises(oktest.UnsupportedResponseObjectError, errmsg % 'content type')
+        #
+        def fn(): ok ("foo")._resp.header('Content-Type', 'text/plain')
+        ok (fn).raises(oktest.UnsupportedResponseObjectError, errmsg % 'header')
+        #
+        def fn(): ok ("foo")._resp.body(["hello!"])
+        ok (fn).raises(oktest.UnsupportedResponseObjectError, errmsg % 'binary body')
+        #
+        def fn(): ok ("foo")._resp.json({"status":"OK"})
+        ok (fn).raises(oktest.UnsupportedResponseObjectError, errmsg % 'content type')
+
 
 
 if __name__ == '__main__':
